@@ -1,4 +1,5 @@
 import numpy as np
+import math
 try:
     import torch
 except ImportError:
@@ -44,7 +45,7 @@ def translate_labellings(from_labels,to_labels,subsample_size='none',preserve_si
     min_num_labs = min(num_from_labs,num_to_labs)
     if abs(num_from_labs-num_to_labs) >= min_num_labs/2 or min_num_labs==1:
         warnings.warn(f"You're translating a labelling with {num_from_labs} different labels into one with {num_to_labs}")
-    trans_dict, leftovers = get_trans_dict(from_labels,to_labels,subsample_size,preserve_sizes)
+    trans_dict = get_trans_dict(from_labels,to_labels,subsample_size,preserve_sizes)
     return np.array([trans_dict[l] for l in from_labels])
 
 def get_trans_dict(from_labels,to_labels,subsample_size='none',preserve_sizes=False):
@@ -62,30 +63,43 @@ def get_trans_dict(from_labels,to_labels,subsample_size='none',preserve_sizes=Fa
     to_labels_compressed, tdt, _ = compress_labels(to_labels)
     reverse_tdf = {v:k for k,v in tdf.items()}
     reverse_tdt = {v:k for k,v in tdt.items()}
-    if num_from_labs <= num_to_labs:
-        trans_dict = get_fanout_trans_dict(from_labels_compressed,to_labels_compressed,subsample_size)
-        leftovers = set([x for x in unique_labels(to_labels_compressed) if x not in trans_dict.values()])
-    else:
-        trans_dict,leftovers = get_fanin_trans_dict(from_labels_compressed,to_labels_compressed,subsample_size,preserve_sizes)
-    if not len(trans_dict) + len(leftovers) == max(num_from_labs,num_to_labs): set_trace()
-    # Account for the possible changes in the above compression
-    trans_dict = {reverse_tdf[k]:reverse_tdt[v] for k,v in trans_dict.items()}
-    if preserve_sizes and num_from_labs > num_to_labs:
+    cost_matrix = compute_cost_matrix(from_labels_compressed,to_labels_compressed,subsample_size)
+    if preserve_sizes:
+        trans_dict = simple_get_trans_dict_from_cost_mat(cost_matrix)
+        trans_dict = {reverse_tdf[k]:reverse_tdt[v] for k,v in trans_dict.items()}
+        leftovers = set([x for x in range(num_from_labs) if x not in trans_dict.keys()])
         for i in leftovers:
             tl = reverse_tdf[i]
             assert tl not in trans_dict.keys()
             missing_target = min([i for i in range(num_from_labs) if i not in trans_dict.values()])
             trans_dict[tl]=missing_target
-        if not set(trans_dict.keys()) == unique_labels(from_labels): set_trace()
-
+    else:
+        trans_dict = get_trans_dict_from_cost_mat(cost_matrix)
+        trans_dict = {reverse_tdf[k]:reverse_tdt[v] for k,v in trans_dict.items()}
+    if not len(trans_dict) == num_from_labs:
+        breakpoint()
     trans_dict[-1] = -1
-    return trans_dict,leftovers
+    return trans_dict
 
-def get_fanout_trans_dict(from_labels,to_labels,subsample_size):
+def simple_get_trans_dict_from_cost_mat(cost_matrix):
+    row_ind, col_ind = linear_sum_assignment(cost_matrix)
+    return {k:v for k,v in enumerate(col_ind)}
+
+def get_trans_dict_from_cost_mat(cost_matrix):
+    m,n = cost_matrix.shape # m is from, n is to
+    if m <= n:
+        return simple_get_trans_dict_from_cost_mat(cost_matrix)
+    else: # tall thin, bad because some rows won't be assigned
+        num_replications_needed = math.ceil(m/n)
+        cost_matrix = np.tile(cost_matrix,(1,num_replications_needed))
+        row_ind, col_ind = linear_sum_assignment(cost_matrix)
+        return {r:col_ind[r]%n for r in row_ind}
+
+def compute_cost_matrix(from_labels,to_labels,subsample_size):
     unique_from_labels = [l for l in unique_labels(from_labels) if l != -1]
     unique_to_labels = [l for l in unique_labels(to_labels) if l != -1]
     if subsample_size == 'none':
-        cost_matrix = np.array([[label_assignment_cost(from_labels,to_labels,l1,l2) for l2 in unique_to_labels] for l1 in unique_from_labels])
+        return np.array([[label_assignment_cost(from_labels,to_labels,l1,l2) for l2 in unique_to_labels] for l1 in unique_from_labels])
     else:
         num_trys = 0
         while True:
@@ -98,45 +112,7 @@ def get_fanout_trans_dict(from_labels,to_labels,subsample_size):
                 from_labels_subsample = from_labels[sample_indices]
                 to_labels_subsample = to_labels[sample_indices]
             if unique_labels(from_labels_subsample) == set(unique_from_labels) and unique_labels(to_labels_subsample) == set(unique_to_labels): break
-        cost_matrix = np.array([[label_assignment_cost(from_labels_subsample,to_labels_subsample,l1,l2) for l2 in unique_from_labels] for l1 in unique_from_labels])
-    row_ind, col_ind = linear_sum_assignment(cost_matrix)
-    if len(col_ind) != len(set(from_labels[from_labels != -1])):
-        raise TranslationError(f"then translation cost matrix is the wrong size for some reason")
-    trans_dict = {l:col_ind[l] for l in unique_labels(from_labels)}
-    return trans_dict
-
-def get_fanin_trans_dict(from_labels,to_labels,subsample_size,preserve_sizes):
-    unique_from_labels = [l for l in unique_labels(from_labels) if l != -1]
-    unique_to_labels = [l for l in unique_labels(to_labels) if l != -1]
-    if subsample_size == 'none':
-        cost_matrix = np.array([[label_assignment_cost(to_labels,from_labels,l1,l2) for l2 in unique_from_labels] for l1 in unique_to_labels])
-    else:
-        while True: # Keep trying random indices unitl you reach one that contains all labels
-            if subsample_size > len(from_labels):
-                from_labels_subsample,to_labels_subsample = from_labels,to_labels
-            else:
-                sample_indices = np.random.choice(range(from_labels.shape[0]),subsample_size,replace=False)
-                from_labels_subsample = from_labels[sample_indices]
-                to_labels_subsample = to_labels[sample_indices]
-            if [l for l in unique_labels(from_labels_subsample) if l != -1] == unique_from_labels and [l for l in unique_labels(to_labels_subsample) if l != -1] == unique_to_labels: break
-        cost_matrix = np.array([[label_assignment_cost(to_labels_subsample,from_labels_subsample,l1,l2) for l2 in unique_from_labels] for l1 in unique_to_labels])
-    row_ind, col_ind = linear_sum_assignment(cost_matrix)
-    if len(col_ind) != get_num_labels(to_labels):
-        raise TranslationError(f"then translation cost matrix is the wrong size for some reason")
-    cl = col_ind.tolist()
-    trans_dict = {f:cl.index(f) for f in cl}
-    num_trys = 0
-    while True:
-        num_trys += 1
-        if num_trys > len(unique_from_labels): set_trace()
-        untranslated = [i for i in unique_from_labels if i not in trans_dict.keys()]
-        unique_untranslated = unique_labels(untranslated)
-        if len(untranslated) == 0 or preserve_sizes: break
-        # Now assign the additional, unassigned items
-        cost_matrix2 = np.array([[label_assignment_cost(from_labels,to_labels,l1,l2) for l2 in unique_to_labels] for l1 in unique_untranslated])
-        row_ind2, col_ind2 = linear_sum_assignment(cost_matrix2)
-        for u,t in zip(untranslated,col_ind2): trans_dict[u]=t
-    return trans_dict, unique_untranslated
+        return np.array([[label_assignment_cost(from_labels_subsample,to_labels_subsample,l1,l2) for l2 in unique_from_labels] for l1 in unique_from_labels])
 
 def get_confusion_mat(labels1,labels2):
     if max(labels1) != max(labels2):
@@ -150,7 +126,6 @@ def get_confusion_mat(labels1,labels2):
     return confusion_matrix
 
 def debable(labellings,pivot,subsample_size='none'):
-    #labellings_list.sort(key=lambda x: x.max())
     if isinstance(labellings,np.ndarray):
         if labellings.ndim != 2:
             raise TranslationError(f"If debabling array, it should have 2 dimensions, but here it has {labellings.ndim}")
